function y = Cheby2Poly(x, M)
% Evaluate an M'th order Chebyshev polynomial of the second kind
% y = Cheby2Poly(x, M)
% y <-  Value of the polynomial at the points specified by x
% x  -> Vector of x values
% M  -> Polynomial order

% $Id: Cheby2Poly.m,v 1.2 2025/01/29 16:29:25 pkabal Exp $

Nx = length(x);
y = NaN(size(x));
for k = 1:Nx
  xk = x(k);
  if (xk > -1 && xk < +1)
    y(k) = sin((M+1) * acos(xk)) / sin(acos(xk));
  elseif (xk == 1)
    y(k) = M+1;
  elseif (xk > 1)
    y(k) = sinh((M+1) * acosh(xk)) / sinh(acosh(xk));
  elseif (xk == -1)
    y(k) = (-1)^M * (M+1);
  else
    y(k) = (-1)^M * sinh((M+1) * acosh(-xk)) / sinh(acosh(-xk));
  end
end

end
