function [RMSErr, g] = RMSErrOptGain(x, y)
% Measure the root mean-square error between vectors x and g*y, where g is a
% gain chosen to minimize the mean-square difference between x and g*y. The
% returned value is
%   RMSErr = sqrt(sum((x - g*y).^2) / N),
% where
%   g = sum(x .* y) / sum(y .* y).
%
% RMSErr <- Root mean-square error
% g      <-  Gain applied to y
% x       -> Reference vector
% y       -> Test vector (scalar or same length as x)

% $Id: RMSErrOptGain.m,v 1.1 2024/12/11 20:25:48 pkabal Exp $

if (length(y) ~= 1 && length(y) ~= length(x))
  error('RMSErrNorm: input vectors must be of the same length');
end

x = x(:);
y = y(:);
Syy = y' * y;
if (Syy == 0)
  g = 0;    % y is all zeros, any value of g works
else
  Sxy = x' * y;
  g = Sxy / Syy;
end

% E2 = (x-gy)' (x-gy)
%    = x'x - 2g x'y + g^2 y'y
% dE2/dg = -2x'y + 2g y'y
%        = 0
% g = (x'y) / (y'y)
RMSErr = rms(x - g * y);

end
