function tUSPolyCalc()
% Test US polynomial calculations

% $Id: tUSPolyCalc.m,v 1.7 2025/04/22 17:56:50 pkabal Exp $

SysInfo = true;
CheckDiaryFile = StartDiaryFile(SysInfo);

SavePath = addpath('..', '../extras');

% Test cases
Nw = [23, 24, 239, 240, 1023, 1024];
RdB = 60;
alphaV = [-0.5, 0, 0.5, 1, 1.5, 2];

for alpha = alphaV
  fprintf('----- -----\n')
  for N = Nw
    % Find x0 which gives RdB of sidelobe attenuation
    x0 = USWinRdB(N, RdB, alpha);

    RdB = FindUSWinRdB(N, x0, alpha);
    sigma = FindUSWinMLW(N, x0, alpha);
    fprintf('N = %d, alpha = %g, x0 = %.7g, RdB = %.3g, sigma = %.3g\n', ...
            N, alpha, x0, RdB, sigma);
    RunPolyTest(N-1, x0, alpha);
    fprintf('\n');
  end
end

path(SavePath);
CheckDiaryFile();

end

% ----- -----
function RunPolyTest(M, x0, alpha)

% x values from -x0 to x0
x = linspace(-x0, x0, M+1);

PU = USPoly(x, M, alpha);
PU = PU / PU(end);
SymErr = CheckSym(PU);
if (SymErr)
  fprintf('  Symmetry Error USPoly: %.3g\n', SymErr);
end
  
if (alpha == 0 || alpha == 1)
  PUx = USPoly0(x, M, alpha);
  PUx = PUx/PUx(end);
  d = max(abs(PUx - PU));
  if (d ~= 0)
    d
  end
  switch alpha
  case 0
    PC = Cheby1Poly(x, M);
  case 1
    PC = Cheby2Poly(x, M);
    otherwise
  end
  PC = PC / PC(end);
  SymErr = CheckSym(PC);
  if (SymErr ~= 0)
    fprintf('  Symmetry Error Chebyshev polynomial: %.3g\n', SymErr);
  end
  fprintf('  Max diff USPoly vs Cheby: %.3g\n', max(abs(PU - PC)));
end

end

% ----- -----
function SymErr = CheckSym(P)

M = length(P);
if (mod(M, 2) == 0)
  SymErr = max(abs(P + fliplr(P)));
else
  SymErr = max(abs(P - fliplr(P)));
end

end

% ----- -----
function y = USPoly0(x, M, alpha)

assert(alpha == 0 || alpha ==1);

yp = 1;         % m = 0
if (alpha == 0)
  y = 1/2;
else
  y = 1;  % m = 1, y scaled by 1/2x
end

if (M == 0)
  y = yp;
end

x24 = 4*x.^2;
meven = false;
for m = 2:M
  meven = ~meven;
  ypp = yp;
  yp = y;
  if (meven)
    y = x24.*yp - ypp;  % m even, actual value
  else
    y = yp - ypp;       % m odd, scaled by 1/2x
  end
end

% Fix y for M odd if RedPoly is false
if (~meven)
  y = 2*x .* y;
end

if (alpha == 0 && M > 0)
  y = 2*y / M;
end

end
