function tUSWinR()
% Test window recurrence calculation

SysInfo = true;
CheckDiaryFile = StartDiaryFile(SysInfo);

SavePath = addpath('..', '../extras');

NV = 1:8;
alphaV = [0, 1, 1.5];
sigma = 1.5;

for alpha = alphaV
  fprintf('sigma = %g, alpha = %g\n', sigma, alpha);
  NN = length(NV);
  corrER1 = NaN(1, NN);
  corrER1A = NaN(1, NN);
  corrER2 = NaN(1, NN);
  if (alpha == 0 || alpha == 1)
    corrChebyER2 = NaN(1, NN);
  end
  for i = 1:NN
    N = NV(i);
    % Find x0 which gives RdB of sidelobe attenuation
    if (N == 1)
      x0 = USWinMLW(N, NaN, alpha);
    elseif (N == 2)
      x0 = USWinMLW(N, 2/N, alpha);
    else
      x0 = USWinMLW(N, sigma, alpha);
    end
    S = FindUSWinMLW(N, x0, alpha);
    if (N == 1)
      assert(isnan(S));
    elseif (N == 2)
      assert(S == 2/N);
    else
      assert(abs(S-sigma) < 1e-14);
    end

    href = USWin(N, x0, alpha, false);
    corrER1(i) = CorrError(href, USWinR1(N, x0, alpha, false));
    corrER1A(i) = CorrError(href, USWinR1A(N, x0, alpha, false));
    corrER2(i) = CorrError(href, USWinR2(N, x0, alpha, false));
    if (alpha == 0 || alpha == 1)
      corrChebyER2(i) = CorrError(href, USChebyWinR2(N, x0, alpha, false));
    end
  end
  PrintCorr('USWinR1', corrER1);
  PrintCorr('USWinR1A', corrER1A);
  PrintCorr('USWinR2', corrER2);
  if (alpha == 0 || alpha == 1)
    PrintCorr('USChebyWinR2', corrChebyER2);
  end
end

path(SavePath);
CheckDiaryFile();

end

% ----- -----
function corrE = CorrError(href, h)

% Normalized cross-correlation error
%  cErr = 1 - (hr' * h) / (hr' * hr)
%       = (hr' * (hr - h)) / (hr' * hr)

corrE = (href' * (href - h)) / (href' * href);

end

% ----- -----
function PrintCorr(Banner, corrE)

NN = length(corrE);

fprintf('  %s\n', Banner);
fprintf('      N:')
for i = 1:NN
  fprintf('%5d ', i);
end
fprintf('\n');

fprintf('  CorrE: ')
for i = 1:NN
  fprintf('%5.2f ', corrE(i) * 1e16);
end
fprintf('(x 1e-16)\n');

end