function tUSWinTime()
% Time the different procedures for US windows

% $Id: tUSWinTime.m,v 1.2 2025/04/14 13:45:51 pkabal Exp $

SysInfo = true;
CheckDiaryFile = StartDiaryFile(SysInfo);

SavePath = addpath('..', '../extras');

NV = [240, 1024];

RdB = 60;
alpha = 1;

fn = {@USWin, @USWinR1, @USWinR1A, @USWinR2, @USChebyWinR2, @USWinSeries};
name = {'USWin', 'USWinR1', 'USWinR1A', 'USWinR2', ...
        'USChebyWinR2', 'USWinSeries'};
M = length(fn);

t = NaN(length(NV), M);
x0 = NaN(size(NV));

for i = 1:length(NV)

  N = NV(i);

  % DT window
  % Find x0 which gives RdB sidelobe suppression
  x0(i) = USWinMLW(N, RdB, alpha);

  for k = 1:M
    t(i,k) = TimeUSWin(fn{k}, N, x0(i), alpha);
  end

end

% CT Window
% *** For alpha = 1 ***
dBVInv = @(x) 10.^(x/20);
R = dBVInv(RdB);

ftan = @(theta)(tan(theta)-theta);
theta = fzero(ftan, [pi+eps, 3*pi/2-eps]);

fbeta = @(beta)(sinh(beta) - beta*R*abs(cos(theta)));
beta = fzero(fbeta, [eps,50]);

tref = t(1, 4);
for i = 1:length(NV)
  N = NV(i);

  fprintf('DT US Window\n');
  fprintf('  N = %d, alpha = %g, x0 = %.7g, R = %.3g dB\n', ...
          N, alpha, x0(i), RdB);

  fprintf('CT US Window\n');
  fprintf('  N = %d, alpha = %g, beta = %.5g, R = %.3g dB\n', ...
          N, alpha, beta, RdB);

  fprintf('\n      Routine  rel time  abs time\n');
  fprintf(  ' ------------  --------  --------\n');
  for k = 1:M
    fprintf('%13s %7.3f %8.3f ms\n', name{k}, t(i,k)/tref, 1000*t(i,k));  
  end

  fprintf('\n');
  tc = TimeUSWin(@CTUSWinN, N, beta, alpha);
  fprintf('%13s %7.3f %8.2f ms\n', 'CTUSWinN', tc/tref, 1000*tc');

  if (i ~= length(NV))
    fprintf('\n');
  end
end

path(SavePath);
CheckDiaryFile();

end

% ----- -----
function t = TimeUSWin(fn, N, arg, alpha)

NK = 1000;

fn(N, arg, alpha);
tic();
for k = 1:NK
  fn(N, arg, alpha);
end
t = toc() / NK;

end
